@@ -86,9 +86,8 @@
 
 enum {
         MODE_UNDEFINED = 0,
-        MODE_TIMED_LOGIN,
         MODE_SELECTION,
-        MODE_AUTHENTICATION,
+        MODE_AUTHENTICATION
 };
 
 enum {
@@ -110,11 +109,10 @@ struct GdmGreeterLoginWindowPrivate
         gboolean         banner_message_enabled;
         guint            gconf_cnxn;
 
-        guint            last_mode;
         guint            dialog_mode;
 
         gboolean         user_list_disabled;
-        guint            num_queries;
+        gboolean         show_cancel_button;
 
         gboolean         timed_login_already_enabled;
         gboolean         timed_login_enabled;
@@ -166,14 +164,14 @@ set_busy (GdmGreeterLoginWindow *login_window)
         GdkCursor *cursor;
 
         cursor = gdk_cursor_new (GDK_WATCH);
-        gdk_window_set_cursor (GTK_WIDGET (login_window)->window, cursor);
+        gdk_window_set_cursor (gtk_widget_get_window (GTK_WIDGET (login_window)), cursor);
         gdk_cursor_unref (cursor);
 }
 
 static void
 set_ready (GdmGreeterLoginWindow *login_window)
 {
-        gdk_window_set_cursor (GTK_WIDGET (login_window)->window, NULL);
+        gdk_window_set_cursor (gtk_widget_get_window (GTK_WIDGET (login_window)), NULL);
 }
 
 static void
@@ -198,11 +196,11 @@ set_focus (GdmGreeterLoginWindow *login_window)
 
         entry = GTK_WIDGET (gtk_builder_get_object (GDM_GREETER_LOGIN_WINDOW (login_window)->priv->builder, "auth-prompt-entry"));
 
-        gdk_window_focus (GTK_WIDGET (login_window)->window, GDK_CURRENT_TIME);
+        gdk_window_focus (gtk_widget_get_window (GTK_WIDGET (login_window)), GDK_CURRENT_TIME);
 
-        if (GTK_WIDGET_REALIZED (entry) && ! GTK_WIDGET_HAS_FOCUS (entry)) {
+        if (gtk_widget_get_realized (entry) && ! gtk_widget_has_focus (entry)) {
                 gtk_widget_grab_focus (entry);
-        } else if (GTK_WIDGET_REALIZED (login_window->priv->user_chooser) && ! GTK_WIDGET_HAS_FOCUS (login_window->priv->user_chooser)) {
+        } else if (gtk_widget_get_realized (login_window->priv->user_chooser) && ! gtk_widget_has_focus (login_window->priv->user_chooser)) {
                 gtk_widget_grab_focus (login_window->priv->user_chooser);
         }
 }
@@ -283,7 +281,7 @@ static gboolean
 timed_login_timer (GdmGreeterLoginWindow *login_window)
 {
         set_sensitive (login_window, FALSE);
-        set_message (login_window, _("Automatically logging in..."));
+        set_message (login_window, _("Automatically logging in…"));
 
         g_debug ("GdmGreeterLoginWindow: timer expired");
         _gdm_greeter_login_window_set_interactive (login_window, TRUE);
@@ -335,6 +333,19 @@ show_widget (GdmGreeterLoginWindow *login_window,
         }
 }
 
+static void
+sensitize_widget (GdmGreeterLoginWindow *login_window,
+                  const char            *name,
+                  gboolean               sense)
+{
+        GtkWidget *widget;
+
+        widget = GTK_WIDGET (gtk_builder_get_object (login_window->priv->builder, name));
+        if (widget != NULL) {
+                gtk_widget_set_sensitive (widget, sense);
+        }
+}
+
 static void
 on_login_button_clicked_answer_query (GtkButton             *button,
                                       GdmGreeterLoginWindow *login_window)
@@ -395,55 +406,22 @@ set_log_in_button_mode (GdmGreeterLoginWindow *login_window,
         }
 }
 
-static gboolean
-user_chooser_has_no_user (GdmGreeterLoginWindow *login_window)
-{
-        guint num_items;
-
-        num_items = gdm_chooser_widget_get_number_of_items (GDM_CHOOSER_WIDGET (login_window->priv->user_chooser));
-        g_debug ("GdmGreeterLoginWindow: loaded=%d num_items=%d",
-                 login_window->priv->user_chooser_loaded,
-                 num_items);
-        return (login_window->priv->user_chooser_loaded && num_items == 0);
-}
-
 static void
-maybe_show_cancel_button (GdmGreeterLoginWindow *login_window)
+adjust_other_login_visibility(GdmGreeterLoginWindow *login_window)
 {
-        gboolean show;
-
-        show = FALSE;
-
-        /* only show the cancel button if there is something to go
-           back to */
-
-        switch (login_window->priv->dialog_mode) {
-        case MODE_SELECTION:
-                /* should never have anything to return to from here */
-                show = FALSE;
-                break;
-        case MODE_TIMED_LOGIN:
-                /* should always have something to return to from here */
-                show = TRUE;
-                break;
-        case MODE_AUTHENTICATION:
-                if (login_window->priv->num_queries > 1) {
-                        /* if we are inside a pam conversation past
-                           the first step */
-                        show = TRUE;
-                } else {
-                        if (login_window->priv->user_list_disabled || user_chooser_has_no_user (login_window)) {
-                                show = FALSE;
-                        } else {
-                                show = TRUE;
-                        }
-                }
-                break;
-        default:
-                g_assert_not_reached ();
+        if (! login_window->priv->user_chooser_loaded) {
+                return;
         }
 
-        show_widget (login_window, "cancel-button", show);
+        if (login_window->priv->dialog_mode != MODE_SELECTION) {
+                return;
+        }
+
+        if (gdm_chooser_widget_get_number_of_items (GDM_CHOOSER_WIDGET (login_window->priv->user_chooser)) == 0) {
+                set_log_in_button_mode (login_window, LOGIN_BUTTON_ANSWER_QUERY);
+        } else {
+                set_log_in_button_mode (login_window, LOGIN_BUTTON_HIDDEN);
+        }
 }
 
 static void
@@ -459,35 +437,57 @@ switch_mode (GdmGreeterLoginWindow *login_window,
         /* we want to run this even if we're supposed to
            be in the mode already so that we reset everything
            to a known state */
-        if (login_window->priv->dialog_mode != number) {
-                login_window->priv->last_mode = login_window->priv->dialog_mode;
-                login_window->priv->dialog_mode = number;
-        }
+        login_window->priv->dialog_mode = number;
 
         default_name = NULL;
 
         switch (number) {
         case MODE_SELECTION:
                 set_log_in_button_mode (login_window, LOGIN_BUTTON_HIDDEN);
-                break;
-        case MODE_TIMED_LOGIN:
-                set_log_in_button_mode (login_window, LOGIN_BUTTON_TIMED_LOGIN);
+
+                show_widget (login_window, "cancel-button", FALSE);
+
+                show_widget (login_window, "auth-input-box", FALSE);
+
+                sensitize_widget (login_window, "disconnect-button", FALSE);
+
+                /*
+                 * Although the cancel button is not shown in the selection
+                 * dialog, the show_cancel_button flag manages when the
+                 * cancel button is shown on subsequent PAM queries.  Set the
+                 * flag to true initially since the button should appear on
+                 * the first query, since the button takes the user back to
+                 * selection mode.
+                 */
+                login_window->priv->show_cancel_button = TRUE;
                 break;
         case MODE_AUTHENTICATION:
-                set_log_in_button_mode (login_window, LOGIN_BUTTON_ANSWER_QUERY);
+                show_widget (login_window, "disconnect-button", FALSE);
+                default_name = "log-in-button";
                 break;
         default:
                 g_assert_not_reached ();
         }
 
-        show_widget (login_window, "auth-input-box", FALSE);
-        maybe_show_cancel_button (login_window);
+        box = GTK_WIDGET (gtk_builder_get_object (login_window->priv->builder,
+                          "buttonbox"));
+        gtk_button_box_set_layout (GTK_BUTTON_BOX (box),
+                                   (number == MODE_SELECTION) ?
+                                    GTK_BUTTONBOX_SPREAD :
+                                    GTK_BUTTONBOX_END );
+
+        if (default_name != NULL) {
+                GtkWidget *widget;
+
+                widget = GTK_WIDGET (gtk_builder_get_object (login_window->priv->builder, default_name));
+                gtk_widget_grab_default (widget);
+        }
 
         /*
          * The rest of this function sets up the user list, so just return if
          * the user list is disabled.
          */
-        if (login_window->priv->user_list_disabled && number != MODE_TIMED_LOGIN) {
+        if (login_window->priv->user_list_disabled) {
                 return;
         }
 
@@ -524,56 +524,35 @@ delete_entry_text (GtkWidget *entry)
         gtk_entry_set_text (GTK_ENTRY (entry), "");
 }
 
-static gboolean
-can_jump_to_authenticate (GdmGreeterLoginWindow *login_window)
-{
-        gboolean res;
-
-        if (!login_window->priv->user_chooser_loaded) {
-                res = FALSE;
-        } else if (login_window->priv->user_list_disabled) {
-                res = (login_window->priv->timed_login_username == NULL);
-        } else {
-                res = user_chooser_has_no_user (login_window);
-        }
-
-        return res;
-}
-
 static void
-reset_dialog (GdmGreeterLoginWindow *login_window,
-              guint                  dialog_mode)
+reset_dialog (GdmGreeterLoginWindow *login_window)
 {
         GtkWidget  *entry;
         GtkWidget  *label;
 
-        g_debug ("GdmGreeterLoginWindow: Resetting dialog to mode %u", dialog_mode);
+        g_debug ("GdmGreeterLoginWindow: Resetting dialog");
         set_busy (login_window);
         set_sensitive (login_window, FALSE);
 
-        login_window->priv->num_queries = 0;
+        login_window->priv->show_cancel_button = FALSE;
 
-        if (dialog_mode == MODE_SELECTION) {
-                if (login_window->priv->timed_login_enabled) {
-                        gdm_chooser_widget_set_item_timer (GDM_CHOOSER_WIDGET (login_window->priv->user_chooser),
-                                                           GDM_USER_CHOOSER_USER_AUTO, 0);
-                        remove_timed_login_timeout (login_window);
-                        login_window->priv->timed_login_enabled = FALSE;
-                }
-                _gdm_greeter_login_window_set_interactive (login_window, FALSE);
+        if (login_window->priv->timed_login_enabled) {
+                gdm_chooser_widget_set_item_timer (GDM_CHOOSER_WIDGET (login_window->priv->user_chooser),
+                                                   GDM_USER_CHOOSER_USER_AUTO, 0);
+                remove_timed_login_timeout (login_window);
+                login_window->priv->timed_login_enabled = FALSE;
+        }
+        _gdm_greeter_login_window_set_interactive (login_window, FALSE);
 
-                g_signal_handlers_block_by_func (G_OBJECT (login_window->priv->user_chooser),
-                                                 G_CALLBACK (on_user_unchosen), login_window);
-                gdm_user_chooser_widget_set_chosen_user_name (GDM_USER_CHOOSER_WIDGET (login_window->priv->user_chooser), NULL);
-                g_signal_handlers_unblock_by_func (G_OBJECT (login_window->priv->user_chooser),
-                                                   G_CALLBACK (on_user_unchosen), login_window);
+        g_signal_handlers_block_by_func (G_OBJECT (login_window->priv->user_chooser),
+                                         G_CALLBACK (on_user_unchosen), login_window);
+        gdm_user_chooser_widget_set_chosen_user_name (GDM_USER_CHOOSER_WIDGET (login_window->priv->user_chooser), NULL);
+        g_signal_handlers_unblock_by_func (G_OBJECT (login_window->priv->user_chooser),
+                                           G_CALLBACK (on_user_unchosen), login_window);
 
-                if (login_window->priv->start_session_handler_id > 0) {
-                        g_signal_handler_disconnect (login_window, login_window->priv->start_session_handler_id);
-                        login_window->priv->start_session_handler_id = 0;
-                }
-
-                set_message (login_window, "");
+        if (login_window->priv->start_session_handler_id > 0) {
+                g_signal_handler_disconnect (login_window, login_window->priv->start_session_handler_id);
+                login_window->priv->start_session_handler_id = 0;
         }
 
         entry = GTK_WIDGET (gtk_builder_get_object (GDM_GREETER_LOGIN_WINDOW (login_window)->priv->builder, "auth-prompt-entry"));
@@ -581,23 +560,22 @@ reset_dialog (GdmGreeterLoginWindow *login_window,
         delete_entry_text (entry);
 
         gtk_entry_set_visibility (GTK_ENTRY (entry), TRUE);
+        set_message (login_window, "");
 
         label = GTK_WIDGET (gtk_builder_get_object (GDM_GREETER_LOGIN_WINDOW (login_window)->priv->builder, "auth-prompt-label"));
         gtk_label_set_text (GTK_LABEL (label), "");
 
-        if (can_jump_to_authenticate (login_window)) {
-                /* If we don't have a user list jump straight to authenticate */
-                g_debug ("GdmGreeterLoginWindow: jumping straight to authenticate");
+        if (login_window->priv->user_list_disabled) {
                 switch_mode (login_window, MODE_AUTHENTICATION);
         } else {
-                switch_mode (login_window, dialog_mode);
+                switch_mode (login_window, MODE_SELECTION);
         }
 
         set_sensitive (login_window, TRUE);
         set_ready (login_window);
         set_focus (GDM_GREETER_LOGIN_WINDOW (login_window));
         update_banner_message (login_window);
-
+        adjust_other_login_visibility (login_window);
         if (gdm_chooser_widget_get_number_of_items (GDM_CHOOSER_WIDGET (login_window->priv->user_chooser)) >= 1) {
                 gdm_chooser_widget_propagate_pending_key_events (GDM_CHOOSER_WIDGET (login_window->priv->user_chooser));
         }
@@ -607,7 +585,7 @@ static void
 do_cancel (GdmGreeterLoginWindow *login_window)
 {
         /* need to wait for response from backend */
-        set_message (login_window, _("Cancelling..."));
+        set_message (login_window, _("Cancelling…"));
         set_busy (login_window);
         set_sensitive (login_window, FALSE);
         g_signal_emit (login_window, signals[CANCELLED], 0);
@@ -618,15 +596,15 @@ gdm_greeter_login_window_ready (GdmGreeterLoginWindow *login_window)
 {
         g_return_val_if_fail (GDM_IS_GREETER_LOGIN_WINDOW (login_window), FALSE);
 
-        reset_dialog (login_window, MODE_SELECTION);
+        reset_dialog (login_window);
 
         set_sensitive (GDM_GREETER_LOGIN_WINDOW (login_window), TRUE);
         set_ready (GDM_GREETER_LOGIN_WINDOW (login_window));
         set_focus (GDM_GREETER_LOGIN_WINDOW (login_window));
 
         /* If the user list is disabled, then start the PAM conversation */
-        if (can_jump_to_authenticate (login_window)) {
-                g_debug ("Starting PAM conversation since user list disabled or no local users");
+        if (login_window->priv->user_list_disabled) {
+                g_debug ("Starting PAM conversation since user list disabled");
                 g_signal_emit (G_OBJECT (login_window), signals[USER_SELECTED],
                                0, GDM_USER_CHOOSER_USER_OTHER);
                 g_signal_emit (login_window, signals[BEGIN_VERIFICATION], 0);
@@ -640,8 +618,7 @@ gdm_greeter_login_window_reset (GdmGreeterLoginWindow *login_window)
 {
         g_return_val_if_fail (GDM_IS_GREETER_LOGIN_WINDOW (login_window), FALSE);
 
-        g_debug ("GdmGreeterLoginWindow: got reset");
-        reset_dialog (login_window, MODE_SELECTION);
+        reset_dialog (GDM_GREETER_LOGIN_WINDOW (login_window));
 
         return TRUE;
 }
@@ -668,28 +645,31 @@ gdm_greeter_login_window_problem (GdmGreeterLoginWindow *login_window,
         g_debug ("GdmGreeterLoginWindow: problem: %s", text);
 
         set_message (GDM_GREETER_LOGIN_WINDOW (login_window), text);
-        gdk_window_beep (GTK_WIDGET (login_window)->window);
+        gdk_window_beep (gtk_widget_get_window (GTK_WIDGET (login_window)));
 
         return TRUE;
 }
 
 static void
-request_timed_login (GdmGreeterLoginWindow *login_window)
+handle_request_timed_login (GdmGreeterLoginWindow *login_window)
 {
-        g_debug ("GdmGreeterLoginWindow: requesting timed login");
-
-        gtk_widget_show (login_window->priv->user_chooser);
-
         if (login_window->priv->dialog_mode != MODE_SELECTION) {
-                reset_dialog (login_window, MODE_SELECTION);
+                reset_dialog (login_window);
         }
+        gdm_user_chooser_widget_set_show_user_auto (GDM_USER_CHOOSER_WIDGET (login_window->priv->user_chooser), TRUE);
 
         if (!login_window->priv->timed_login_already_enabled) {
                 gdm_user_chooser_widget_set_chosen_user_name (GDM_USER_CHOOSER_WIDGET (login_window->priv->user_chooser),
                                                               GDM_USER_CHOOSER_USER_AUTO);
         }
+}
 
-        login_window->priv->timed_login_already_enabled = TRUE;
+static void
+on_request_timed_login_after_users_loaded (GdmUserChooserWidget  *user_chooser,
+                                           GdmGreeterLoginWindow *login_window)
+{
+        g_debug ("Users now loaded, handling timed login request");
+        handle_request_timed_login (login_window);
 }
 
 void
@@ -701,20 +681,24 @@ gdm_greeter_login_window_request_timed_login (GdmGreeterLoginWindow *login_windo
 
         g_debug ("GdmGreeterLoginWindow: requested automatic login for user '%s' in %d seconds", username, delay);
 
-        g_free (login_window->priv->timed_login_username);
+        if (login_window->priv->timed_login_username != NULL) {
+                login_window->priv->timed_login_already_enabled = TRUE;
+                g_free (login_window->priv->timed_login_username);
+        } else {
+                login_window->priv->timed_login_already_enabled = FALSE;
+        }
         login_window->priv->timed_login_username = g_strdup (username);
         login_window->priv->timed_login_delay = delay;
 
-        /* add the auto user right away so we won't trigger a mode
-           switch to authenticate when the user list is disabled */
-        gdm_user_chooser_widget_set_show_user_auto (GDM_USER_CHOOSER_WIDGET (login_window->priv->user_chooser), TRUE);
-
-        /* if the users aren't loaded then we'll handle it in when they are */
         if (login_window->priv->user_chooser_loaded) {
                 g_debug ("Handling timed login request since users are already loaded.");
-                request_timed_login (login_window);
+                handle_request_timed_login (login_window);
         } else {
                 g_debug ("Waiting to handle timed login request until users are loaded.");
+                g_signal_connect (login_window->priv->user_chooser,
+                                  "loaded",
+                                  G_CALLBACK (on_request_timed_login_after_users_loaded),
+                                  login_window);
         }
 }
 
@@ -755,6 +739,21 @@ gdm_greeter_login_window_start_session_when_ready (GdmGreeterLoginWindow *login_
         }
 }
 
+static void
+_show_cancel_button (GdmGreeterLoginWindow *login_window)
+{
+        /*
+         * If show_cancel_button is false, this is because the face browser
+         * is disabled and the first query is showing.  It does not make sense
+         * to display the cancel button when showing the initial query and
+         * the face browser is disabled.  Set the flag to true so that the
+         * button is shown on following queries.
+         */
+        show_widget (login_window, "cancel-button",
+                     login_window->priv->show_cancel_button);
+        login_window->priv->show_cancel_button = TRUE;
+}
+
 gboolean
 gdm_greeter_login_window_info_query (GdmGreeterLoginWindow *login_window,
                                      const char            *text)
@@ -764,8 +763,7 @@ gdm_greeter_login_window_info_query (GdmGreeterLoginWindow *login_window,
 
         g_return_val_if_fail (GDM_IS_GREETER_LOGIN_WINDOW (login_window), FALSE);
 
-        login_window->priv->num_queries++;
-        maybe_show_cancel_button (login_window);
+        _show_cancel_button (login_window);
 
         g_debug ("GdmGreeterLoginWindow: info query: %s", text);
 
@@ -796,8 +794,7 @@ gdm_greeter_login_window_secret_info_query (GdmGreeterLoginWindow *login_window,
 
         g_return_val_if_fail (GDM_IS_GREETER_LOGIN_WINDOW (login_window), FALSE);
 
-        login_window->priv->num_queries++;
-        maybe_show_cancel_button (login_window);
+        _show_cancel_button (login_window);
 
         entry = GTK_WIDGET (gtk_builder_get_object (GDM_GREETER_LOGIN_WINDOW (login_window)->priv->builder, "auth-prompt-entry"));
         delete_entry_text (entry);
@@ -891,8 +888,8 @@ cancel_button_clicked (GtkButton             *button,
 static void
 on_user_chooser_visibility_changed (GdmGreeterLoginWindow *login_window)
 {
-        g_debug ("GdmGreeterLoginWindow: Chooser visibility changed");
         update_banner_message (login_window);
+        adjust_other_login_visibility (login_window);
 }
 
 static void
@@ -901,60 +898,10 @@ on_users_loaded (GdmUserChooserWidget  *user_chooser,
 {
         g_debug ("GdmGreeterLoginWindow: users loaded");
         login_window->priv->user_chooser_loaded = TRUE;
-
         update_banner_message (login_window);
+        adjust_other_login_visibility (login_window);
 
-        if (!login_window->priv->user_list_disabled) {
-                gtk_widget_show (login_window->priv->user_chooser);
-        }
-
-        if (login_window->priv->timed_login_username != NULL
-            && !login_window->priv->timed_login_already_enabled) {
-                request_timed_login (login_window);
-        } else if (can_jump_to_authenticate (login_window)) {
-                /* jump straight to authenticate */
-                g_debug ("GdmGreeterLoginWindow: jumping straight to authenticate");
-
-                switch_mode (login_window, MODE_AUTHENTICATION);
-
-                g_debug ("Starting PAM conversation since no local users");
-                g_signal_emit (G_OBJECT (login_window), signals[USER_SELECTED],
-                               0, GDM_USER_CHOOSER_USER_OTHER);
-                g_signal_emit (login_window, signals[BEGIN_VERIFICATION], 0);
-        }
-}
-
-static void
-choose_user (GdmGreeterLoginWindow *login_window,
-             const char            *user_name)
-{
-        guint mode;
-
-        g_assert (user_name != NULL);
-
-        g_signal_emit (G_OBJECT (login_window), signals[USER_SELECTED],
-                       0, user_name);
-
-        mode = MODE_AUTHENTICATION;
-        if (strcmp (user_name, GDM_USER_CHOOSER_USER_OTHER) == 0) {
-                g_signal_emit (login_window, signals[BEGIN_VERIFICATION], 0);
-        } else if (strcmp (user_name, GDM_USER_CHOOSER_USER_GUEST) == 0) {
-                /* FIXME: handle guest account stuff */
-        } else if (strcmp (user_name, GDM_USER_CHOOSER_USER_AUTO) == 0) {
-                g_signal_emit (login_window, signals[BEGIN_AUTO_LOGIN], 0,
-                               login_window->priv->timed_login_username);
-
-                login_window->priv->timed_login_enabled = TRUE;
-                restart_timed_login_timeout (login_window);
-
-                /* just wait for the user to select language and stuff */
-                mode = MODE_TIMED_LOGIN;
-                set_message (login_window, _("Select language and click Log In"));
-        } else {
-                g_signal_emit (login_window, signals[BEGIN_VERIFICATION_FOR_USER], 0, user_name);
-        }
-
-        switch_mode (login_window, mode);
+        gdm_chooser_widget_activate_if_one_item (GDM_CHOOSER_WIDGET (login_window->priv->user_chooser));
 }
 
 static void
@@ -970,7 +917,29 @@ on_user_chosen (GdmUserChooserWidget  *user_chooser,
                 return;
         }
 
-        choose_user (login_window, user_name);
+        g_signal_emit (G_OBJECT (login_window), signals[USER_SELECTED],
+                       0, user_name);
+
+        if (strcmp (user_name, GDM_USER_CHOOSER_USER_OTHER) == 0) {
+                g_signal_emit (login_window, signals[BEGIN_VERIFICATION], 0);
+        } else if (strcmp (user_name, GDM_USER_CHOOSER_USER_GUEST) == 0) {
+                /* FIXME: handle guest account stuff */
+        } else if (strcmp (user_name, GDM_USER_CHOOSER_USER_AUTO) == 0) {
+                g_signal_emit (login_window, signals[BEGIN_AUTO_LOGIN], 0,
+                               login_window->priv->timed_login_username);
+
+                login_window->priv->timed_login_enabled = TRUE;
+                restart_timed_login_timeout (login_window);
+
+                /* just wait for the user to select language and stuff */
+                set_log_in_button_mode (login_window, LOGIN_BUTTON_TIMED_LOGIN);
+                set_message (login_window, _("Select language and click Log In"));
+        } else {
+                g_signal_emit (login_window, signals[BEGIN_VERIFICATION_FOR_USER], 0, user_name);
+        }
+
+        switch_mode (login_window, MODE_AUTHENTICATION);
+
         g_free (user_name);
 }
 
@@ -1027,7 +996,6 @@ file_read_one_line (const char *filename)
 
         if (fgets (buf, sizeof (buf), f) == NULL) {
                 g_warning ("Unable to read from file %s", filename);
-                goto out;
         }
 
         line = g_strdup (buf);
@@ -1202,6 +1170,10 @@ load_theme (GdmGreeterLoginWindow *login_window)
                                  G_CALLBACK (on_user_chooser_visibility_changed),
                                  login_window);
 
+        if (!login_window->priv->user_list_disabled) {
+                gtk_widget_show (login_window->priv->user_chooser);
+        }
+
         login_window->priv->auth_banner_label = GTK_WIDGET (gtk_builder_get_object (login_window->priv->builder, "auth-banner-label"));
         /*make_label_small_italic (login_window->priv->auth_banner_label);*/
 
@@ -1239,8 +1211,7 @@ gdm_greeter_login_window_key_press_event (GtkWidget   *widget,
         login_window = GDM_GREETER_LOGIN_WINDOW (widget);
 
         if (event->keyval == GDK_Escape) {
-                if (login_window->priv->dialog_mode == MODE_AUTHENTICATION
-                    || login_window->priv->dialog_mode == MODE_TIMED_LOGIN) {
+                if (login_window->priv->dialog_mode == MODE_AUTHENTICATION) {
                         do_cancel (GDM_GREETER_LOGIN_WINDOW (widget));
                 }
         }
@@ -1261,19 +1232,20 @@ gdm_greeter_login_window_size_request (GtkWidget      *widget,
                 GTK_WIDGET_CLASS (gdm_greeter_login_window_parent_class)->size_request (widget, requisition);
         }
 
-        if (!GTK_WIDGET_REALIZED (widget)) {
+        if (!gtk_widget_get_realized (widget)) {
                 return;
         }
 
         screen = gtk_widget_get_screen (widget);
-        monitor = gdk_screen_get_monitor_at_window (screen, widget->window);
+        monitor = gdk_screen_get_monitor_at_window (screen, gtk_widget_get_window (widget));
         gdk_screen_get_monitor_geometry (screen, monitor, &area);
 
-        gtk_widget_get_child_requisition (GTK_BIN (widget)->child, &child_requisition);
+        gtk_widget_get_child_requisition (gtk_bin_get_child (GTK_BIN (widget)), &child_requisition);
         *requisition = child_requisition;
 
-        requisition->width += 2 * GTK_CONTAINER (widget)->border_width;
-        requisition->height += 2 * GTK_CONTAINER (widget)->border_width;
+        guint border_width = gtk_container_get_border_width (GTK_CONTAINER (widget));
+        requisition->width += 2 * border_width;
+        requisition->height += 2 * border_width;
 
         /* Make width be at least 33% screen width
          * and height be at most 80% of screen height
@@ -1283,7 +1255,10 @@ gdm_greeter_login_window_size_request (GtkWidget      *widget,
 
        /* Don't ever shrink window width
         */
-       requisition->width = MAX (requisition->width, widget->allocation.width);
+        GtkAllocation widget_allocation;
+        gtk_widget_get_allocation (widget, &widget_allocation);
+
+        requisition->width = MAX (requisition->width, widget_allocation.width);
 }
 
 static void
@@ -1313,7 +1288,7 @@ update_banner_message (GdmGreeterLoginWindow *login_window)
         } else {
                 char *message = NULL;
                 error = NULL;
-                if (user_chooser_has_no_user (login_window)) {
+                if (login_window->priv->user_chooser_loaded && gdm_chooser_widget_get_number_of_items (GDM_CHOOSER_WIDGET (login_window->priv->user_chooser)) == 0) {
                         message = gconf_client_get_string (login_window->priv->client, KEY_BANNER_MESSAGE_TEXT_NOCHOOSER, &error);
                         if (error != NULL) {
                                 g_debug("GdmGreeterLoginWindow: unable to get nochooser banner text: %s", error->message);
@@ -1527,6 +1502,7 @@ gdm_greeter_login_window_init (GdmGreeterLoginWindow *login_window)
         GConfClient *client;
         GError      *error;
         gboolean     user_list_disable;
+        gboolean     timed_login_enable;
 
         gdm_profile_start (NULL);
 
@@ -1541,14 +1517,22 @@ gdm_greeter_login_window_init (GdmGreeterLoginWindow *login_window)
          * timed login is also not being used.
          */
         user_list_disable = gconf_client_get_bool (client,
-                                                   KEY_DISABLE_USER_LIST,
-                                                   &error);
+                KEY_DISABLE_USER_LIST, &error);
         if (error != NULL) {
                 g_debug ("GdmUserChooserWidget: unable to get disable-user-list configuration: %s", error->message);
                 g_error_free (error);
         }
 
-        login_window->priv->user_list_disabled = user_list_disable;
+        gdm_settings_client_get_boolean (GDM_KEY_TIMED_LOGIN_ENABLE,
+                                         &timed_login_enable);
+
+        login_window->priv->user_list_disabled = FALSE;
+        if (user_list_disable && !timed_login_enable) {
+                g_debug ("Disabling user list");
+                login_window->priv->user_list_disabled = TRUE;
+        } else {
+                g_debug ("Enabling user list");
+        }
 
         gtk_window_set_title (GTK_WINDOW (login_window), _("Login Window"));
         /*gtk_window_set_opacity (GTK_WINDOW (login_window), 0.85);*/
@@ -1559,6 +1543,7 @@ gdm_greeter_login_window_init (GdmGreeterLoginWindow *login_window)
         gtk_window_set_skip_taskbar_hint (GTK_WINDOW (login_window), TRUE);
         gtk_window_set_skip_pager_hint (GTK_WINDOW (login_window), TRUE);
         gtk_window_stick (GTK_WINDOW (login_window));
+        gtk_window_set_type_hint (GTK_WINDOW (login_window), GDK_WINDOW_TYPE_HINT_DOCK);
         gtk_container_set_border_width (GTK_CONTAINER (login_window), 0);
 
         g_signal_connect (login_window,
